@@ -51,10 +51,12 @@ struct ksmbd_startup_request {
 	__u32	smb2_max_credits;
 	__u32	smbd_max_io_size;	/* smbd read write size */
 	__u32	max_connections;	/* Number of maximum simultaneous connections */
-	__u32   reserved[126];		/* Reserved room */
+	__s8	bind_interfaces_only;
+	__u32   max_ip_connections;	/* Number of maximum connection per ip address */
+	__s8	reserved[499];		/* Reserved room */
 	__u32	ifc_list_sz;
 	__s8	____payload[];
-};
+} __attribute__((packed));
 
 #define KSMBD_STARTUP_CONFIG_INTERFACES(s)	((s)->____payload)
 
