@@ -31,7 +31,7 @@
 
 static GHashTable	*ch_table;
 static GRWLock		ch_table_lock;
-static GArray		*domain_entries;
+static GPtrArray	*domain_entries;
 static gchar		*domain_name;
 static int		num_domain_entries;
 
@@ -86,6 +86,8 @@ static int samr_connect5_invoke(struct ksmbd_rpc_pipe *pipe)
 
 	if (ndr_read_uniq_vstring_ptr(dce, &server_name))
 		return KSMBD_RPC_EINVALID_PARAMETER;
+	ndr_free_uniq_vstring_ptr(&server_name);
+
 	// Access mask
 	if (ndr_read_int32(dce, NULL))
 		return KSMBD_RPC_EINVALID_PARAMETER;
@@ -141,14 +143,14 @@ int samr_ndr_write_domain_array(struct ksmbd_rpc_pipe *pipe)
 	int i, ret = 0;
 
 	for (i = 0; i < num_domain_entries; i++) {
-		gpointer entry;
+		char *entry;
 		size_t name_len;
 
 		ret = ndr_write_int32(dce, i);
 		if (ret)
 			return ret;
-		entry = g_array_index(domain_entries, gpointer, i);
-		name_len = strlen((char *)entry);
+		entry = g_ptr_array_index(domain_entries, i);
+		name_len = strlen(entry);
 		ret = ndr_write_int16(dce, name_len*2);
 		if (ret)
 			return ret;
@@ -165,10 +167,10 @@ int samr_ndr_write_domain_array(struct ksmbd_rpc_pipe *pipe)
 	}
 
 	for (i = 0; i < num_domain_entries; i++) {
-		gpointer entry;
+		char *entry;
 
-		entry = g_array_index(domain_entries,  gpointer, i);
-		ret = ndr_write_string(dce, (char *)entry);
+		entry = g_ptr_array_index(domain_entries, i);
+		ret = ndr_write_string(dce, entry);
 		if (ret)
 			return ret;
 	}
@@ -251,15 +253,16 @@ static int samr_lookup_domain_return(struct ksmbd_rpc_pipe *pipe)
 		return KSMBD_RPC_EBAD_DATA;
 
 	for (i = 0; i < num_domain_entries; i++) {
-		gpointer entry;
+		char *entry;
 
-		entry = g_array_index(domain_entries, gpointer, i);
-		if (!strcmp(STR_VAL(dce->sm_req.name), (char *)entry)) {
+		entry = g_ptr_array_index(domain_entries, i);
+		if (!strcmp(STR_VAL(dce->sm_req.name), entry)) {
 			smb_init_domain_sid(&sid);
 			if (smb_write_sid(dce, &sid))
 				return KSMBD_RPC_EBAD_DATA;
 		}
 	}
+	ndr_free_uniq_vstring_ptr(&dce->sm_req.name);
 
 	return KSMBD_RPC_OK;
 }
@@ -390,6 +393,9 @@ static int samr_open_user_return(struct ksmbd_rpc_pipe *pipe)
 		return KSMBD_RPC_EBAD_FID;
 	ch->refcount++;
 
+	if (!ch->user)
+		return KSMBD_RPC_EBAD_FID;
+
 	if (dce->sm_req.rid != ch->user->uid)
 		return KSMBD_RPC_EBAD_FID;
 
@@ -425,7 +431,7 @@ static int samr_query_user_info_return(struct ksmbd_rpc_pipe *pipe)
 	if (gethostname(hostname, NAME_MAX))
 		return KSMBD_RPC_ENOMEM;
 
-	home_dir_len = 2 + strlen(hostname) + 1 + strlen(ch->user->name);
+	home_dir_len = 2 + strlen(hostname) + 1 + strlen(ch->user->name) + 1;
 
 	home_dir = g_try_malloc0(home_dir_len);
 	if (!home_dir)
@@ -731,6 +737,9 @@ static int samr_query_security_return(struct ksmbd_rpc_pipe *pipe)
 	if (!ch)
 		return KSMBD_RPC_EBAD_FID;
 
+	if (!ch->user)
+		return KSMBD_RPC_EBAD_FID;
+
 	curr_offset = dce->offset;
 	dce->offset += 16;
 	if (build_sec_desc(dce, &sec_desc_len, ch->user->uid))
@@ -999,67 +1008,63 @@ int rpc_samr_write_request(struct ksmbd_rpc_pipe *pipe)
 	return samr_invoke(pipe);
 }
 
-static int rpc_samr_add_domain_entry(char *name)
+static void rpc_samr_add_domain_entry(char *name)
 {
-	char *domain_string;
-
-	domain_string = strdup(name);
-	if (!domain_string)
-		return KSMBD_RPC_ENOMEM;
-
-	domain_entries = g_array_append_val(domain_entries, domain_string);
+	g_ptr_array_add(domain_entries, g_strdup(name));
 	num_domain_entries++;
-
-	return 0;
 }
 
-static void domain_entry_free(void *v)
+void rpc_samr_init(void)
 {
-	char **entry = v;
+	if (!domain_name) {
+		char hostname[NAME_MAX];
 
-	free(*entry);
-}
+		/*
+		 * ksmbd supports the standalone server and
+		 * uses the hostname as the domain name.
+		 */
+		if (gethostname(hostname, NAME_MAX))
+			abort();
 
-int rpc_samr_init(void)
-{
-	char hostname[NAME_MAX];
+		domain_name = g_ascii_strup(hostname, -1);
+	}
+
+	if (!domain_entries) {
+		domain_entries = g_ptr_array_new_with_free_func(g_free);
+		rpc_samr_add_domain_entry(domain_name);
+		rpc_samr_add_domain_entry("Builtin");
+	}
 
-	ch_table = g_hash_table_new(g_str_hash, g_str_equal);
 	if (!ch_table)
-		return -ENOMEM;
-	domain_entries = g_array_new(0, 0, sizeof(void *));
-	if (!domain_entries)
-		return -ENOMEM;
+		ch_table = g_hash_table_new(g_str_hash, g_str_equal);
+}
 
-	g_array_set_clear_func(domain_entries, domain_entry_free);
+static void samr_ch_clear_table(void)
+{
+	struct connect_handle *ch;
+	GHashTableIter iter;
 
-	/*
-	 * ksmbd supports the standalone server and
-	 * uses the hostname as the domain name.
-	 */
-	if (gethostname(hostname, NAME_MAX))
-		return -ENOMEM;
-
-	domain_name = g_ascii_strup(hostname, strlen(hostname));
-	if (rpc_samr_add_domain_entry(domain_name))
-		return -ENOMEM;
-	if (rpc_samr_add_domain_entry("Builtin"))
-		return -ENOMEM;
-	g_rw_lock_init(&ch_table_lock);
-	return 0;
+	g_rw_lock_writer_lock(&ch_table_lock);
+	ghash_for_each(ch, ch_table, iter)
+		g_free(ch);
+	g_rw_lock_writer_unlock(&ch_table_lock);
 }
 
 void rpc_samr_destroy(void)
 {
 	if (ch_table) {
+		samr_ch_clear_table();
 		g_hash_table_destroy(ch_table);
 		ch_table = NULL;
 	}
-	g_rw_lock_clear(&ch_table_lock);
-	num_domain_entries = 0;
-	g_free(domain_name);
+
 	if (domain_entries) {
-		g_array_free(domain_entries, 1);
+		g_ptr_array_free(domain_entries, 1);
 		domain_entries = NULL;
 	}
+
+	num_domain_entries = 0;
+
+	g_free(domain_name);
+	domain_name = NULL;
 }
